function [P,valid] = ComputePfromH(H,Adj)

% computes transition probabilities from H
% Must check valid output parameter to know if the result is ok
% H: matrix of mutul hitting times
% A: adjacency matrix (to enforce p_ij is 0 if there is no edge)

% some error checking first
if size(H) ~= size(H')
    error('Matrix H must be square');
end

if size(Adj) ~= size(H)
    error('Adj and H must have the same size');
end

n = size(H,1);

for i =1:n
    if H(i,i) ~= 0
        error('Self-hitting times must be 0');
    end
end

global epsilon;

N = n^2;

AP = zeros(size(H));
Lin = zeros(N,2);
P = zeros(size(H));
K=1;
for i = 1:n
    for j = 1:n
        AP(i,j) = K;
        Lin(K,1) = i;
        Lin(K,2) = j;
        K = K+1;
    end
end

b = zeros(N,1);
A = zeros(N,N);

% first determine A
for eq = 1:N
    i=  Lin(eq,1);
    j = Lin(eq,2);
    if i==j % No equation for H(i,i); rather constraint to sum up to 1
        b(eq) = 1;
        for k = 1:n
            A(eq,AP(i,k))=1;
        end
    else
        b(eq) = H(i,j)-1;
        for k = 1:n
            if k ~= j
                pos = AP(i,k);
                if k==i
                    A(eq,pos) = H(i,j);
                else
                    A(eq,pos) = H(k,j);
                end
            end
        end
    end
end


equationsToGo = zeros(N,1);

for i = 1:n
    for j=1:n
        if Adj(i,j)==0 % no edge? Then set equation p_ij = 0
           eq = AP(i,j);
           equationsToGo(eq) = 1;
           A(:,eq) = zeros(N,1);
        end
    end
end

% now kickout useless cols in A and rwos in b

toKeep = N - sum(equationsToGo);
newA = zeros(N,toKeep);


cur = 1;
for i = 1:numel(equationsToGo)
    if equationsToGo(i) == 0 % equation to keep
        newA(:,cur) = A(:,i);
       
        cur = cur + 1;
    end
end



% This is known to work
% for i = 1:n
%     for j=1:n
%         if Adj(i,j)==0 % no edge? Then set equation p_ij = 0
%             if (i==j) % special case: p_ii = 0;
%                 % must now look for a row I can zero out...
%                 for k = 1:n % finds a row that won't be zeroed out on its own
%                    if ( Adj(i,k) > 0 ) && ( k ~= i )
%                        eq = AP(i,k);
%                        b(eq) = 0;
%                        A(eq,:) = zeros(1,N);
%                        A(eq,AP(i,j)) = 1;
%                        break
%                    end
%                 end
%             else
%                 eq = AP(i,j);
%                 b(eq) = 0;
%                 A(eq,:) = zeros(1,N);
%                 A(eq,eq) = 1;
%             end
%         end
%     end
% end



result = linsolve(newA,b);

Res = zeros(1,N);
cur = 1;
for j = 1:numel(equationsToGo)
    if equationsToGo(j)==0
        Res(j) = result(cur);
        cur = cur+1;
    end
end

for i = 1:N
    P(Lin(i,1),Lin(i,2))=Res(i);
end

if (max(max(P)) > (1+epsilon)) || (min(min(P)) < (-epsilon))
    valid = 0;
else
    valid = 1;
end

rowP = sum(P,2);
if (min(rowP) < 1-epsilon) || ( min(rowP) > 1+epsilon)
    valid = 0;
end


% tested: works;
