function ADJ=adj_from_grid(GRIDZ,show)
% creates an adkacency binary matrix given a matrix that looks like an
% occupancy grid
% basilico@elet.polimi.it, AIRLab, DEI, PoliMI
%
% INPUT:
% GRIDZ: a matrix that looks like an occupancy grid; example
% this grid (O = free cell, X = occupied cell)
% O - O - X
% |   |   |
% O - O - O
% |   |   |
% O - X - O
% is described this matrix
% 1 1 0
% 1 1 1
% 1 0 1
% show: binary flag, if ==1 displays the grid in a figure
%%
% OUTPUT:
% the adjacency matrix describing the input grid. Wrt to the example, this
% will be the naming convention for nodes
% 1 - 2   X
% |   |   
% 3 - 4 - 5
% |       |
% 6   X   7
% in the adjacency matrix we'll have ADJ(1,3)=1 and ADJ(4,7)=0




[r,c] = size(GRIDZ);
%% build the adjacency matrix
numberedGRIDZ=GRIDZ;
curNode=0;
for i=1:r
    for j=1:c
        if numberedGRIDZ(i,j)==1
            curNode=curNode+1;
            numberedGRIDZ(i,j)=curNode;
        end
    end
end

% curNode is now the number of cells
ADJ = zeros(curNode,curNode);
%%{
for i=1:r
    for j=1:c
        node=numberedGRIDZ(i,j);
        
        if node >= 1 
            % up
            if(i>1)
                if numberedGRIDZ(i-1,j) >= 1
                    ADJ(node,numberedGRIDZ(i-1,j))=1;
                    ADJ(numberedGRIDZ(i-1,j),node)=1;
                    %if ~batchMode
                        %disp(['Node ' num2str(node) ' is adjacent to Node ' num2str(numberedGRIDZ(i-1,j)) '(up)']);
                    %end
                end
            end
            % down
            if(i<r)
                if numberedGRIDZ(i+1,j) >= 1
                    ADJ(node,numberedGRIDZ(i+1,j))=1;
                    ADJ(numberedGRIDZ(i+1,j),node)=1;
                    %if ~batchMode
                        %disp(['Node ' num2str(node) ' is adjacent to Node ' num2str(numberedGRIDZ(i+1,j)) '(down)']);
                    %end    
                end            
            end            
            
            % left
            if(j>1)
                if numberedGRIDZ(i,j-1) >= 1
                    ADJ(node,numberedGRIDZ(i,j-1))=1;
                    ADJ(numberedGRIDZ(i,j-1),node)=1;
                    %if ~batchMode
                        %disp(['Node ' num2str(node) ' is adjacent to Node ' num2str(numberedGRIDZ(i,j-1)) '(left)']);
                    %end                        
                end            
            end                        
            % right        
            if(j<c)
                if numberedGRIDZ(i,j+1) >= 1
                    ADJ(node,numberedGRIDZ(i,j+1))=1;
                    ADJ(numberedGRIDZ(i,j+1),node)=1;
                    %if ~batchMode
                        %disp(['Node ' num2str(node) ' is adjacent to Node ' num2str(numberedGRIDZ(i,j+1)) '(right)']);
                    %end                        
                end            
            end                        
        end
    end
end

if show

    for i=1:r
        for j=1:c
            %if ismember(numberedGRIDZ(i,j),targets)
             %   GRIDZ(i,j) = 0.5;
            %end
        end
    end
    imagesc((1:c)+0.5,(1:r)+0.5,GRIDZ);  
    colormap(hot);
    axis equal
    set(gca,'XTick',1:(c+1),'YTick',1:(r+1),'XLim',[1 c+1],'YLim',[1 r+1],'GRIDLineStyle','-','XGRID','on','YGRID','on');

    % scale the image (GRIDZ) and save as jpeg
    MM=[];
    scale=20;
    for i=1:c
        M=[];
            for j=1:r
                M=[M repmat(GRIDZ(j,i),scale,scale)];
            end
        MM=[MM; M];
    end
end