% Uncomment to get repeatable runs
%RandStream.setDefaultStream(RandStream('mt19937ar','seed',40));

function main(confFile)

%setup global variables

%clear all

global K; % number of robots
global N; % number of vertices in the graph
global T; % temporal deadline
global DeltaT; % discretization step


global SFunctionHandle; % this is the function handle

global distance
global parent

global DebugInfo; % should print out lost of stuff?
global BigInstance; % should I do approximate computation for nchoosek?
global LowerUpperApproximation; % 1 = LowerApproximation, 2 = UpperApproximation

global SAMPLING_STRATEGY;

global TMin;
global TMax;


if nargin==1 % we need to read a configuration file
    
    [keynames,values]=textread(confFile,'%s=%d');
    myStruct = cell2struct(num2cell(values), keynames);
    
    SIMULATIONTYPE = myStruct.SIMULATIONTYPE;
    DATASET = myStruct.DATASET;
    DEPLOYMENTSITE = myStruct.DEPLOYMENTSITE;
    VISUALIZEMAP = myStruct.VISUALIZEMAP;
    DebugInfo = myStruct.DebugInfo;
    BigInstance = myStruct.BigInstance;
    LowerUpperApproximation = myStruct.LowerUpperApproximation;
    K = myStruct.K;
    T = myStruct.T;
    DeltaT = myStruct.DeltaT;
    MonteCarloTrials = myStruct.MonteCarloTrials; % number of trials for statistics on repeated deployments
    step = myStruct.step;
    LowerMultiplier = myStruct.LowerMultiplier;
    UpperMultiplier = myStruct.UpperMultiplier;
    SUFFIX = myStruct.SUFFIX;
    SAMPLING_STRATEGY = myStruct.SAMPLING_STRATEGY;
    TMin = myStruct.TMin;
    TMax = myStruct.TMax;
    
else % setup paramters internally
    
    SIMULATIONTYPE = 7;
    DATASET = 2;
    DEPLOYMENTSITE = 1;
    VISUALIZEMAP = 1;
    DebugInfo = 0;
    BigInstance = 1;
    LowerUpperApproximation = 1;
    K = 50;
    T = 680;
    DeltaT = 1;
    MonteCarloTrials = 100; % number of trials for statistics on repeated deployments
    step = 1;
    LowerMultiplier = 1;
    UpperMultiplier = 10;
    SUFFIX = 1;
    SAMPLING_STRATEGY = 1;
    TMin = 10;
    TMax = 40;
end

%clear all
close all
clc


VISUALIZE_MAP = 1;
%DebugInfo = 1;

% Decides what we're going to do
% 1: simualtes a deployment for given parameters of K and T with different
% functions Si
% 2: simualtes a deployment for given parameters of K and T with the same
% function S
% 3: compute bound (formula 1 in the write up) for single setup
% 4: compute bound for a range in the case all S are the same
% 5: MonteCarlo simulation of various deployments for the simplified case
% 6: MonteCarlo simulation of various deployments for the general case with
% different S functions
% 7: compute bound for a range in the case all S are different


% DATASETS
% 1 - simple map
% 2 - sdr40


% for DATASET 1 (for TMin = 10,TMax = 40) a critical time is 180 because
% max(distance)=7
% for DATASET 2 (for TMin = 10,TMax = 40)



% s is the deployment site




% setup graph and costs
switch DATASET
    case 1
        [A,C] = LoadAndSemplifyGraph('Maps/testmapadjacency_matrix.txt','Maps/testmapcoordinates.txt',0);
        GoalDestinations = 2:1:size(A,2);
        if (VISUALIZEMAP == 1) || ( SIMULATIONTYPE==1) || ( SIMULATIONTYPE>=6)
            bitmap = DisplayGraphOnMap(A,C,GoalDestinations,DEPLOYMENTSITE,'Maps/testmapinverted.bmp',1);
        end
        if SIMULATIONTYPE==4
            if LowerUpperApproximation==1
                savefilename = 'simplemap_bound_equalS_lower.mat';
            else
                savefilename = 'simplemap_bound_equalS_upper.mat';
            end
        elseif SIMULATIONTYPE==5
            if SAMPLING_STRATEGY == 1
                savefilename = 'simplemap_deployment_equalS.mat';
            else
                savefilename = 'simplemap_deployment_equalS_improved.mat';
            end
        elseif SIMULATIONTYPE==6
            if SAMPLING_STRATEGY == 1
                savefilename = sprintf('simplemap_deployment_differentS_%d.mat',SUFFIX);
            else
                savefilename = sprintf('simplemap_deployment_differentS_%d_improved.mat',SUFFIX);
            end
        elseif SIMULATIONTYPE==7
            if LowerUpperApproximation==1
                savefilename = 'simplemap_bound_differentS_lower.mat';
            else
                savefilename = 'simplemap_bound_differentS_upper.mat';
            end
        end
    case 2
        [A,C] = LoadAndSemplifyGraph('Maps/sdr40adjacency_matrix.txt','Maps/sdr40coordinates.txt',1,50);
       % GoalDestinations = [3 8 10 55 58 68 107 64 102 66 100 113 61 117 131 120  140 121 125 9 134 128 133 16 ...
       %     105 103 67 78 139 141 137 126 80 90 75 77 81 46 45 40 36 34 26 23 ];
        GoalDestinations = [32 33 2 4 6 26 24 22 21 19 16 13 52 43 11 42 ];
        if (VISUALIZEMAP == 1) || ( SIMULATIONTYPE==1)|| ( SIMULATIONTYPE>=6)
            bitmap = DisplayGraphOnMap(A,C,GoalDestinations,DEPLOYMENTSITE,'Maps/sdr40_final_inverted.bmp',1);
        end
        if SIMULATIONTYPE==4
            if LowerUpperApproximation==1
                savefilename = 'sdr40_bound_equalS_lower.mat';
            else
                savefilename = 'sdr40_bound_equalS_upper.mat';
            end
        elseif SIMULATIONTYPE==5
            if SAMPLING_STRATEGY == 1
                savefilename = 'sdr40_deployment_equalS.mat';
            else
                savefilename = 'sdr40_deployment_equalS_improved.mat';
            end
        elseif SIMULATIONTYPE==6
            if SAMPLING_STRATEGY == 1
                savefilename = sprintf('sdr40_deployment_differentS_%d.mat',SUFFIX);
            else
                savefilename = sprintf('sdr40_deployment_differentS_%d_improved.mat',SUFFIX);
            end
        elseif SIMULATIONTYPE==7
            if LowerUpperApproximation==1
                savefilename = 'sdr40_bound_differentS_lower.mat';
            else
                savefilename = 'sdr40_bound_differentS_upper.mat';
            end
        end
end


GoalDestinations = sort(GoalDestinations);
N = numel(GoalDestinations);

% bounds for K when considering multiple runs
LowerBoundK = LowerMultiplier*N;
UpperBoundK = UpperMultiplier*N;

%  SFunctionHandle = @SimplifiedTransition;
[distance,parent] = Dijkstra(A,DEPLOYMENTSITE);
% AllocateMassDistribution

switch SIMULATIONTYPE
    case 1
        SFunctionHandle = @SigmoidTransitionProbability;
        [Params,Lookup] = AnnotateGraph(A,C,bitmap);
        RunMission(DEPLOYMENTSITE,A,Params,Lookup,GoalDestinations);
    case 2
        [distance,parent] = Dijkstra(A,DEPLOYMENTSITE);
        SFunctionHandle = @SimplifiedTransition;
        RunMissionSimplified(DEPLOYMENTSITE,A,GoalDestinations);
    case 3
        SFunctionHandle = @SimplifiedTransition;
        successProbability = ComputeBound(A,DEPLOYMENTSITE,GoalDestinations);
    case 4
        SFunctionHandle = @SimplifiedTransition;
        successProbability = zeros(1,numel(LowerBoundK:step:UpperBoundK));
        j=1;
        for K = LowerBoundK:step:UpperBoundK
            %for K = 200:200
            K
            if ( K > 50 ) && ( BigInstance == 0)
                fprintf('Switching to BigInstance for K=%d\n',K);
                BigInstance = 1;
            end
            %ComputeBound(A,DEPLOYMENTSITE,GoalDestinations)
            successProbability(j) = ComputeBound(A,DEPLOYMENTSITE,GoalDestinations);
            j = j+1;
        end
        save(savefilename);
        plot(LowerBoundK:step:UpperBoundK,successProbability)
        save(savefilename);
        if VISUALIZEMAP == 1
            figure
            plot(LowerBoundK:step:UpperBoundK,successProbability);
        end
    case 5
        [distance,parent] = Dijkstra(A,DEPLOYMENTSITE);
        if max(distance) == 10^6
            error('One or more vertices unreachable from the source');
        end
        SFunctionHandle = @SimplifiedTransition;
        successProbabilityDeployment = zeros(1,numel(LowerBoundK:step:UpperBoundK));
        j=1;
        %   Frequency = zeros(numel(GoalDestinations)+1,1);
        %   errorNotChosen = 0;
        %   errorNotReached = 0;
        %   success = 0;
        for K = LowerBoundK:step:UpperBoundK
            K
            for i = 1:MonteCarloTrials
                
                [ChosenDestinations,Successful,Reached] = RunMissionSimplified(DEPLOYMENTSITE,A,GoalDestinations);
                %          for ll = 1:numel(ChosenDestinations)
                %              Frequency(ChosenDestinations(ll)) = Frequency(ChosenDestinations(ll))+1;
                %          end
                if sum(Reached) == N
                    successProbabilityDeployment(j) = successProbabilityDeployment(j) + 1;
                    %             success = success + 1;
                    %         else
                    % log information for failure
                    %             bitarray = zeros(size(GoalDestinations));
                    %                     for jj = 1:numel(ChosenDestinations)
                    %                         bitarray(find(GoalDestinations==ChosenDestinations(jj))) = 1;
                    %                     end
                    %                     if sum(bitarray)==numel(GoalDestinations)
                    %                         errorNotReached = errorNotReached + 1;
                    %                     else
                    %                         errorNotChosen = errorNotChosen + 1;
                    %                     end
                end
            end
            
            successProbabilityDeployment(j) = successProbabilityDeployment(j)/MonteCarloTrials;
            save(savefilename);
            j = j+1;
        end
        save(savefilename);
    case 6
        [distance,parent] = Dijkstra(A,DEPLOYMENTSITE);
        if max(distance) == 10^6
            error('One or more vertices unreachable from the source');
        end
        SFunctionHandle = @SigmoidTransitionProbability;
        [Params,Lookup] = AnnotateGraph(A,C,bitmap);
        successProbabilityDeployment = zeros(1,numel(LowerBoundK:step:UpperBoundK));
        j=1;
        for K = LowerBoundK:step:UpperBoundK
            K
            for i = 1:MonteCarloTrials
                i
                [ChosenDestinations,Successful,Reached] =  RunMission(DEPLOYMENTSITE,A,Params,Lookup,GoalDestinations);
                if sum(Reached) == N
                    successProbabilityDeployment(j) = successProbabilityDeployment(j) + 1;
                end
            end
            successProbabilityDeployment(j) = successProbabilityDeployment(j)/MonteCarloTrials;
            save(savefilename);
            j = j+1;
            
        end
        save(savefilename);
        
    case 7
        [distance,parent] = Dijkstra(A,DEPLOYMENTSITE);
        SFunctionHandle = @SigmoidTransitionProbability;
        [Params,Lookup] = AnnotateGraph(A,C,bitmap);
        successProbability = zeros(1,numel(LowerBoundK:step:UpperBoundK));
        j=1;
        for K = LowerBoundK:step:UpperBoundK
            %for K = 200:200
            K
            if ( K > 50 ) && ( BigInstance == 0)
                fprintf('Switching to BigInstance for K=%d\n',K);
                BigInstance = 1;
            end
            %ComputeBound(A,DEPLOYMENTSITE,GoalDestinations)
            successProbability(j) = ComputeBoundComplex(A,DEPLOYMENTSITE,GoalDestinations,Params,Lookup);
            j = j+1;
        end
        %         save(savefilename);
        %         plot(LowerBoundK:step:UpperBoundK,successProbability)
        if VISUALIZEMAP == 1
            figure
            plot(LowerBoundK:step:UpperBoundK,successProbability);
        end
        figure
        plot(LowerBoundK:step:UpperBoundK,successProbability)
        save(savefilename);
        
end




% figure
% plot(LowerBoundK:UpperBoundK,successProbability)
% grid on
